///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to "CELayoutEditor.cpp".
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

#ifndef _CEIMAGESETCONFIG_H_
#define _CEIMAGESETCONFIG_H_

#include <wx/wx.h>

#include <wx/filename.h>
#include <wx/fileconf.h>

// --------------------------------------------------------------------------------
// ------ Groups and entries definition, they are saved in our config file --------
// --------------------------------------------------------------------------------

#define DEFAULT_CONFIG_FILE_NAME				"CEImagesetEditor"

#define CONFIG_DONT_RUN_INSTALL_WIZARD			"/General/DontRunInstallWizard"

#define CONFIG_COLOR_RED						"/BackgroundColor/ColorRed"
#define CONFIG_COLOR_GREEN						"/BackgroundColor/ColorGreen"
#define CONFIG_COLOR_BLUE						"/BackgroundColor/ColorBlue"

#define CONFIG_LAST_IMAGESET					"/LastLoadedFiles/ImageSet"
#define CONFIG_LAST_SCHEME						"/LastLoadedFiles/Scheme"
#define CONFIG_LAST_FONT						"/LastLoadedFiles/Font"
#define CONFIG_LAST_LAYOUT						"/LastLoadedFiles/Layout"

#define CONFIG_DEFAULT_GROUP_IMAGESET			"/DefaultGroups/ImageSet"
#define CONFIG_DEFAULT_GROUP_FONT				"/DefaultGroups/Font"
#define CONFIG_DEFAULT_GROUP_SCHEME				"/DefaultGroups/Scheme"
#define CONFIG_DEFAULT_GROUP_LOOKNFEELS			"/DefaultGroups/LooknFeels"
#define CONFIG_DEFAULT_GROUP_LAYOUT				"/DefaultGroups/Layout"
#define CONFIG_DEFAULT_GROUP_SCRIPT				"/DefaultGroups/Script"

#define CONFIG_GROUP_ENTRY						"/Groups/"

// Names of the default groups, it's also real directory names used to map those default groups

#define CONFIG_DEFAULT_GROUP_NAME_IMAGESET		"imagesets"
#define CONFIG_DEFAULT_GROUP_NAME_FONT			"fonts"
#define CONFIG_DEFAULT_GROUP_NAME_SCHEME		"schemes"
#define CONFIG_DEFAULT_GROUP_NAME_LOOKNFEELS	"looknfeel"
#define CONFIG_DEFAULT_GROUP_NAME_LAYOUT		"layouts"
#define CONFIG_DEFAULT_GROUP_NAME_SCRIPT		"lua_scripts"


#define CONFIG_FRAME_IMAGESET_ZOOM				"Frame/ImageSetZoom"
#define CONFIG_FRAME_VERTICAL_SASH_POS			"Frame/VerticalSashPos"
#define CONFIG_FRAME_HORIZONTAL_SASH_POS		"Frame/HorizontalSashPos"


// -------------------------------------------------------------------------------

class CEImagesetConfig: public wxFileConfig
{
public:

    ~CEImagesetConfig();

    // Returns pointer to singleton (only one instance)
    static		CEImagesetConfig* GetInstancePtr();
	static void setConfigDir(const wxString configDir);
	static void setConfigName(const wxString configName);
	wxString	getConfigName();
	void		createDefaultEditorConfig();
	void		wizardSaveResourcePaths();
	
private:
  
	// The one-and-only instance of the class.
	static CEImagesetConfig *m_instance;
	
	static wxString			m_configName;
	static wxString			m_configDir;

	// Constructor. Singleton, that's why its private.
	CEImagesetConfig(const wxString& appName, const wxString& ConfigLocalFileName, const wxString& ConfigGlobalFileName);
	
};


#endif // _CEIMAGESETCONFIG_H_
