///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////
#include "DialogDefaultResourceGroups.h"
#include "wxPathCellEditor.h"
#include "CEGUIHelper.h"
//#include "Options.h"
#include <CEGUISystem.h>
#include <CEGUIDefaultResourceProvider.h>

//--------------------------------------------------------------------------------
BEGIN_EVENT_TABLE( DialogDefaultResourceGroups, wxDialog )
EVT_BUTTON( wxID_OK, DialogDefaultResourceGroups::OnOkClick )
EVT_BUTTON( wxID_CANCEL, DialogDefaultResourceGroups::OnCancelClick )
END_EVENT_TABLE()

//--------------------------------------------------------------------------------
DialogDefaultResourceGroups::DialogDefaultResourceGroups( DialogResourceGroups* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style ) :
        m_okButton( 0 ),
        m_cancelButton( 0 )
{
    Create( parent, id, caption, pos, size, style );
}

//--------------------------------------------------------------------------------
bool DialogDefaultResourceGroups::Create( DialogResourceGroups* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    SetExtraStyle( GetExtraStyle() | wxWS_EX_BLOCK_EVENTS );
    wxDialog::Create( parent, id, caption, pos, size, style );

	m_parent = parent;

    CreateControls();

	ShowModal();

    return true;
}

//--------------------------------------------------------------------------------
void DialogDefaultResourceGroups::CreateControls()
{
    DialogDefaultResourceGroups * itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer( wxVERTICAL );
    itemDialog1->SetSizer( itemBoxSizer2 );

    wxStaticBox* itemStaticBoxSizer4Static = new wxStaticBox( itemDialog1, wxID_ANY, wxT( "Default groups" ) );
    wxStaticBoxSizer* itemStaticBoxSizer4 = new wxStaticBoxSizer( itemStaticBoxSizer4Static, wxVERTICAL );
	itemBoxSizer2->Add( itemStaticBoxSizer4, 1, wxGROW | wxALL, 5 );


	// Init groups entries for the combos

	wxArrayString strings;
	wxGrid *parentGrid = m_parent->getGroupsGrid();
	for (int idx=0;idx<parentGrid->GetNumberRows();idx++)
		strings.Add(parentGrid->GetCellValue( idx, 0 ));
	strings.Add(wxT(""));

	itemFlexSizerDefaultGroupList = new wxFlexGridSizer(5,2,0,0);
	itemFlexSizerDefaultGroupList->AddGrowableCol(0);
	itemFlexSizerDefaultGroupList->AddGrowableCol(1);
    itemStaticBoxSizer4->Add( itemFlexSizerDefaultGroupList, 0, wxGROW | wxALL, 5 );

	// ---- ImageSet Default Group

    wxStaticText* itemStaticTextDefaultImageSet = new wxStaticText( itemDialog1, wxID_STATIC, wxT( "ImageSet " ), wxDefaultPosition, wxDefaultSize, 0 );
	itemFlexSizerDefaultGroupList->Add( itemStaticTextDefaultImageSet, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 5 );

	comboImageSet = new wxComboBox (itemDialog1, ID_COMBOBOX_DEFAULT_IMAGESET,wxT("Apple"), wxDefaultPosition, wxDefaultSize,strings, wxCB_READONLY);
	itemFlexSizerDefaultGroupList->Add( comboImageSet, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT, 5 );
	comboImageSet->SetValue(m_parent->getDefaultGroup(DEFAULT_GROUP_IMAGESET));

	// ---- Font Default Group

    wxStaticText* itemStaticTextDefaultFont = new wxStaticText( itemDialog1, wxID_STATIC, wxT( "Font " ), wxDefaultPosition, wxDefaultSize, 0 );
	itemFlexSizerDefaultGroupList->Add( itemStaticTextDefaultFont, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 5 );

	comboFont = new wxComboBox (itemDialog1, ID_COMBOBOX_DEFAULT_FONT,wxT("Apple"), wxDefaultPosition, wxDefaultSize,strings, wxCB_READONLY);
	itemFlexSizerDefaultGroupList->Add( comboFont, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT, 5 );
	comboFont->SetValue(m_parent->getDefaultGroup(DEFAULT_GROUP_FONT));

	// ---- Scheme Default Group

    wxStaticText* itemStaticTextDefaultScheme = new wxStaticText( itemDialog1, wxID_STATIC, wxT( "Scheme " ), wxDefaultPosition, wxDefaultSize, 0 );
	itemFlexSizerDefaultGroupList->Add( itemStaticTextDefaultScheme, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 5 );

	comboScheme = new wxComboBox (itemDialog1, ID_COMBOBOX_DEFAULT_SCHEME,wxT("Apple"), wxDefaultPosition, wxDefaultSize,strings, wxCB_READONLY);
	itemFlexSizerDefaultGroupList->Add( comboScheme, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT, 5 );
	comboScheme->SetValue(m_parent->getDefaultGroup(DEFAULT_GROUP_SCHEME));

	// ---- Looknfeels Default Group

    wxStaticText* itemStaticTextDefaultLooknfeels = new wxStaticText( itemDialog1, wxID_STATIC, wxT( "Looknfeels " ), wxDefaultPosition, wxDefaultSize, 0 );
	itemFlexSizerDefaultGroupList->Add( itemStaticTextDefaultLooknfeels, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 5 );

	comboLooknFeels = new wxComboBox (itemDialog1, ID_COMBOBOX_DEFAULT_LOOKNFEELS,wxT("Apple"), wxDefaultPosition, wxDefaultSize,strings, wxCB_READONLY);
	itemFlexSizerDefaultGroupList->Add( comboLooknFeels, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT, 5 );
	comboLooknFeels->SetValue(m_parent->getDefaultGroup(DEFAULT_GROUP_LOOKNFEELS));

	// ---- Layout Default Group

    wxStaticText* itemStaticTextDefaultLayout = new wxStaticText( itemDialog1, wxID_STATIC, wxT( "Layout " ), wxDefaultPosition, wxDefaultSize, 0 );
	itemFlexSizerDefaultGroupList->Add( itemStaticTextDefaultLayout, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 5 );

	comboLayout = new wxComboBox (itemDialog1, ID_COMBOBOX_DEFAULT_LAYOUT,wxT("Apple"), wxDefaultPosition, wxDefaultSize,strings, wxCB_READONLY);
	itemFlexSizerDefaultGroupList->Add( comboLayout, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT, 5 );
	comboLayout->SetValue(m_parent->getDefaultGroup(DEFAULT_GROUP_LAYOUT));

	// ---- Script Default Group

    wxStaticText* itemStaticTextDefaultScript = new wxStaticText( itemDialog1, wxID_STATIC, wxT( "Script " ), wxDefaultPosition, wxDefaultSize, 0 );
	itemFlexSizerDefaultGroupList->Add( itemStaticTextDefaultScript, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 5 );

	comboScript = new wxComboBox (itemDialog1, ID_COMBOBOX_DEFAULT_SCRIPT,wxT("Apple"), wxDefaultPosition, wxDefaultSize,strings, wxCB_READONLY);
	itemFlexSizerDefaultGroupList->Add( comboScript, 1, wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT, 5 );
	comboScript->SetValue(m_parent->getDefaultGroup(DEFAULT_GROUP_SCRIPT));


    wxStdDialogButtonSizer* itemStdDialogButtonSizer8 = new wxStdDialogButtonSizer;
	itemBoxSizer2->Add( itemStdDialogButtonSizer8, 0, wxALIGN_RIGHT | wxGROW | wxALL, 5 );

    m_okButton = new wxButton( itemDialog1, wxID_OK, wxT( "&OK" ), wxDefaultPosition, wxDefaultSize, 0 );
    m_okButton->SetDefault();
    m_okButton->SetHelpText( wxT( "Accept changes made to resource groups." ) );
    if ( ShowToolTips() )
        m_okButton->SetToolTip( wxT( "Accept changes made to resource groups." ) );
    itemStdDialogButtonSizer8->AddButton( m_okButton );

    m_cancelButton = new wxButton( itemDialog1, wxID_CANCEL, wxT( "&Cancel" ), wxDefaultPosition, wxDefaultSize, 0 );
    m_cancelButton->SetHelpText( wxT( "Cancel all changes made to resource groups." ) );
    if ( ShowToolTips() )
        m_cancelButton->SetToolTip( wxT( "Cancel all changes made to resource groups." ) );
    itemStdDialogButtonSizer8->AddButton( m_cancelButton );

    itemStdDialogButtonSizer8->Realize();
}

//--------------------------------------------------------------------------------
bool DialogDefaultResourceGroups::ShowToolTips()
{
    return true;
}

//--------------------------------------------------------------------------------
void DialogDefaultResourceGroups::OnOkClick( wxCommandEvent& event )
{
	m_parent->setDefaultGroup(DEFAULT_GROUP_IMAGESET,comboImageSet->GetValue());
	m_parent->setDefaultGroup(DEFAULT_GROUP_FONT,comboFont->GetValue());
	m_parent->setDefaultGroup(DEFAULT_GROUP_SCHEME,comboScheme->GetValue());
	m_parent->setDefaultGroup(DEFAULT_GROUP_LOOKNFEELS,comboLooknFeels->GetValue());
	m_parent->setDefaultGroup(DEFAULT_GROUP_LAYOUT,comboLayout->GetValue());
	m_parent->setDefaultGroup(DEFAULT_GROUP_SCRIPT,comboScript->GetValue());

    event.Skip();
}

//--------------------------------------------------------------------------------
void DialogDefaultResourceGroups::OnCancelClick( wxCommandEvent& event )
{

    event.Skip();
}
