///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

#include "ResourceWizardPage.h"
#include "CEImagesetConfig.h"

//--------------------------------------------------------------------------------------

BEGIN_EVENT_TABLE(ResourceWizardPage, wxWizardPageSimple)
    EVT_WIZARD_PAGE_CHANGING(wxID_ANY, ResourceWizardPage::OnWizardPageChanging)
	EVT_RADIOBOX(wxID_ANY, ResourceWizardPage::OnRadioChanging)
    EVT_WIZARD_CANCEL(wxID_ANY, ResourceWizardPage::OnWizardCancel)
END_EVENT_TABLE()


wxString helpTxtOption1 =	wxT("Recommended for first time user:\n")\
							wxT("It configures the editor to use the samples\n");
wxString helpTxtOption2 =	wxT("You select one root folder, which contains CEGUI data.\n")\
							wxT("This folder must copy the samples folder structure:\n")\
							wxT("Meaning it must include the following subfolders:\n")\
							wxT("  - fonts\n")\
							wxT("  - imagesets\n")\
							wxT("  - layouts\n")\
							wxT("  - looknfeel\n")\
							wxT("  - lua_scripts\n")\
							wxT("  - schemes");
wxString helpTxtOption3 =	wxT("You know what you do, you don't need help!\n")\
							wxT("You can manually configure resources later.");


// --------------------------

ResourceWizardPage::ResourceWizardPage(wxWizard *parent) : wxWizardPageSimple(parent)
{

	wxStaticText *text = new wxStaticText(this, wxID_ANY,
		wxT("This tiny wizard will help you\nconfigure CEImageSetEditor Resources.\n")
		wxT("\n")
		wxT("Please Choose your option for configuration:\n"),
			wxPoint(5,5));


    // Auto samples configuration is only available on Win32
	#ifdef WIN32
		static wxString choices[] = { 
			wxT("Use provided sample datas"), 
			wxT("Select a root folder for Resources"), 
			wxT("Don't configure resources")
		};
	#else
		static wxString choices[] = { 
			wxT("Select a root folder for Resources"), 
			wxT("Don't configure resources")
		};
	#endif

    m_radio = new wxRadioBox(this, wxID_ANY, wxT("Resource paths Configuration"),
                             wxDefaultPosition, wxDefaultSize,
                             WXSIZEOF(choices), choices,
                             1, wxRA_SPECIFY_COLS);
    
	m_radio->SetSelection(0);

    mainSizer = new wxBoxSizer(wxVERTICAL);
	mainSizer->Add(text,
        0, // No stretching
        wxALL,
        5 // Border
    );
    
	mainSizer->Add(m_radio,0,wxALL,5);
			
	#if WIN32
		helpText = new wxStaticText(this, wxID_ANY,helpTxtOption1,wxPoint(5,5));
	#else
		helpText = new wxStaticText(this, wxID_ANY,helpTxtOption2,wxPoint(5,5));
	#endif

	mainSizer->Add(helpText,0,wxALL,5);


    SetSizer(mainSizer);
    mainSizer->Fit(this);
}


// ---- wizard event handlers

void ResourceWizardPage::OnWizardCancel(wxWizardEvent& event)
{
    if ( wxMessageBox(wxT("Do you really want to cancel?"), wxT("Question"),
                      wxICON_QUESTION | wxYES_NO, this) != wxYES )
    {
        // not confirmed
        event.Veto();
    }
}

// --------------------------

void ResourceWizardPage::OnWizardPageChanging(wxWizardEvent& event)
{
    int sel = m_radio->GetSelection();

	if ( sel == USESAMPLESRESOURCEPATHS )
	{
		CEImagesetConfig::GetInstancePtr()->wizardSaveResourcePaths();
        return;
	}

    if ( sel == SELECTRESOURCEPATHS )
	{
		wxDirDialog* dataDirDialog = new wxDirDialog(this, 
					wxT("Choose the root CEGUI data folder"), 
					wxT(""), 
					0, 
					wxDefaultPosition);
		dataDirDialog->ShowModal();

		if(dataDirDialog->GetPath().IsEmpty())
		{
			event.Veto();
			return;
		}

		CEImagesetConfig::GetInstancePtr()->setConfigDir(dataDirDialog->GetPath());
		CEImagesetConfig::GetInstancePtr()->wizardSaveResourcePaths();
		return;
	}

    if ( sel == DONTCONFIGURERESOURCEPATHS)
        return;

	//event.Skip();
}

// --------------------------

void ResourceWizardPage::OnRadioChanging(wxCommandEvent& event)
{
	switch (m_radio->GetSelection())
	{
		case USESAMPLESRESOURCEPATHS:
		helpText->SetLabel(helpTxtOption1);
		break;

		case SELECTRESOURCEPATHS:
		helpText->SetLabel(helpTxtOption2);
		break;

		case DONTCONFIGURERESOURCEPATHS:
		helpText->SetLabel(helpTxtOption3);
		break;
	}
	event.Skip();
}
